#include "ty_adc.h"
#include "nrf_drv_saadc.h"
#include "nrf_drv_ppi.h"
#include "nrf_drv_timer.h"




/*********************************************************************
 * LOCAL CONSTANT
 */

/*********************************************************************
 * LOCAL STRUCT
 */

/*********************************************************************
 * LOCAL VARIABLE
 */
#define SAMPLES_IN_BUFFER 1
static const nrf_drv_timer_t m_timer = NRF_DRV_TIMER_INSTANCE(4);
static nrf_saadc_value_t     m_buffer_pool[2][SAMPLES_IN_BUFFER];
static nrf_ppi_channel_t     m_ppi_channel;

static bool volatile ty_adc_flag = false;
static int16_t ty_adc_value = 0;

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */




/*********************************************************
FN: 
*/
static void ty_adc_sampling_event_init(void)
{
    uint32_t ret;

    nrf_drv_timer_config_t timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
    timer_cfg.bit_width = NRF_TIMER_BIT_WIDTH_32;
    ret = nrf_drv_timer_init(&m_timer, &timer_cfg, NULL);
    APP_ERROR_CHECK(ret);

    uint32_t ticks = nrf_drv_timer_ms_to_ticks(&m_timer, 1);
    nrf_drv_timer_extended_compare(&m_timer, NRF_TIMER_CC_CHANNEL0, ticks, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, false);

    ret = nrf_drv_ppi_init();
    APP_ERROR_CHECK(ret);

    ret = nrf_drv_ppi_channel_alloc(&m_ppi_channel);
    APP_ERROR_CHECK(ret);

    uint32_t timer_compare_event_addr = nrf_drv_timer_compare_event_address_get(&m_timer, NRF_TIMER_CC_CHANNEL0);
    uint32_t saadc_sample_task_addr   = nrf_drv_saadc_sample_task_get();
    ret = nrf_drv_ppi_channel_assign(m_ppi_channel, timer_compare_event_addr, saadc_sample_task_addr);
    APP_ERROR_CHECK(ret);
}

/*********************************************************
FN: 
*/
static void ty_adc_callback(nrf_drv_saadc_evt_t const * p_event)
{
    if (p_event->type == NRF_DRV_SAADC_EVT_DONE) {
        uint32_t ret;
        ret = nrf_drv_saadc_buffer_convert(p_event->data.done.p_buffer, SAMPLES_IN_BUFFER);
        APP_ERROR_CHECK(ret);

        ty_adc_value = p_event->data.done.p_buffer[0];
        ty_adc_flag = true;
    }
}

/*********************************************************
FN: 
*/
uint32_t ty_adc_init(ty_adc_t* p_adc)
{
    uint32_t ret;
    
    nrf_drv_saadc_config_t cfg = NRF_DRV_SAADC_DEFAULT_CONFIG;
    cfg.oversample = 6; //64
    ret = nrf_drv_saadc_init(&cfg, ty_adc_callback);
    APP_ERROR_CHECK(ret);

    nrf_saadc_channel_config_t channel_config = NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN7);//NRF_SAADC_INPUT_AIN7-P31
    ret = nrf_drv_saadc_channel_init(p_adc->channel, &channel_config);
    APP_ERROR_CHECK(ret);

    ret = nrf_drv_saadc_buffer_convert(m_buffer_pool[0], SAMPLES_IN_BUFFER);
    APP_ERROR_CHECK(ret);
    ret = nrf_drv_saadc_buffer_convert(m_buffer_pool[1], SAMPLES_IN_BUFFER);
    APP_ERROR_CHECK(ret);
    
    ty_adc_sampling_event_init();
    
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_adc_start(ty_adc_t* p_adc)
{
    nrf_drv_timer_enable(&m_timer);
    
    uint32_t ret = nrf_drv_ppi_channel_enable(m_ppi_channel);
    APP_ERROR_CHECK(ret);
    
    while(!ty_adc_flag){} //It's better to add timeout logic
    
    nrf_drv_timer_disable(&m_timer);
    ty_adc_flag = false;
    p_adc->value = ty_adc_value;
    
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_adc_stop(ty_adc_t* p_adc)
{
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_adc_control(ty_adc_t* p_adc, uint8_t cmd, void* arg)
{
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_adc_uninit(ty_adc_t* p_adc)
{
    nrf_drv_saadc_uninit();
    nrf_drv_saadc_channel_uninit(p_adc->channel);
    nrf_drv_timer_uninit(&m_timer);
    nrf_drv_ppi_uninit();
    return 0;
}







