#include "ty_ble.h"
#include "tuya_ble_api.h"
#include "tuya_ble_ota.h"
#include "tuya_ble_utils.h"
#include "nrf_sdh.h"
#include "nrf_sdh_ble.h"
#include "nrf_ble_gatt.h"
#include "ble_gap.h"
#include "ble_nus.h"
#include "ble_conn_state.h"
#include "app_util.h"
#include "app_error.h"




/*********************************************************************
 * LOCAL CONSTANT
 */
#define     DEFAULT_ADV_DATA    \
            {                   \
                3,              \
                {               \
                    0x02,       \
                    0x01,       \
                    0x06,       \
                },              \
            }
            
#define     DEFAULT_SCAN_RSP    \
            {                   \
                13,              \
                {               \
                    0x0C,       \
                    0x09,       \
                    't', 'y', '_', 'b', 'l', 'e', '_', 'd', 'e', 'm', 'o', \
                },              \
            }

#define     DEFAULT_ADV_PARAM               \
            {                               \
                .adv_interval_min = TY_ADV_INTERVAL,    \
                .adv_interval_max = TY_ADV_INTERVAL,    \
                .adv_type         = 0x01,   \
                .adv_power        = 0x00,   \
                .adv_channal_map  = 0x07,   \
            }

#define     TY_BLE_CONN_CFG_TAG      (1)

/*********************************************************************
 * LOCAL STRUCT
 */

/*********************************************************************
 * LOCAL VARIABLE
 */
static ty_ble_data_t        s_adv_data  = DEFAULT_ADV_DATA;
static ty_ble_data_t        s_scan_rsp  = DEFAULT_SCAN_RSP;
static ty_ble_adv_param_t   s_adv_param = DEFAULT_ADV_PARAM;

//Adv
static uint8_t              m_adv_handle    = BLE_GAP_ADV_SET_HANDLE_NOT_SET;
static ble_gap_adv_params_t m_adv_params    = {0};
static ble_gap_adv_data_t   m_adv_data      = {0};
static bool is_advertising = false;

NRF_BLE_GATT_DEF(m_gatt);
BLE_NUS_DEF(m_nus, NRF_SDH_BLE_TOTAL_LINK_COUNT);
static uint16_t m_conn_handle;

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */




/*********************************************************
FN: 
*/
static void ty_ble_evt_handler(ble_evt_t const * p_ble_evt, void * p_context)
{
    ble_gap_evt_t const * p_gap_evt = &p_ble_evt->evt.gap_evt;
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED: {
            TY_PRINTF("connected");
            
            m_conn_handle = p_gap_evt->conn_handle;
            
            ty_ble_connect_handler();
        } break;

        case BLE_GAP_EVT_DISCONNECTED: {
            TY_PRINTF("disconnect: 0x%02x", p_gap_evt->params.disconnected.reason);
            
            ty_ble_disconnect_handler();
            
            ty_ble_start_adv();
        } break;

        case BLE_GAP_EVT_TIMEOUT: {
            //connect timeout
            if(p_gap_evt->params.timeout.src == BLE_GAP_TIMEOUT_SRC_CONN) {
            }
        } break;

        case BLE_GAP_EVT_PHY_UPDATE_REQUEST: {
            TY_PRINTF("PHY update request.");
            ble_gap_phys_t const phys =
            {
                .rx_phys = BLE_GAP_PHY_AUTO,
                .tx_phys = BLE_GAP_PHY_AUTO,
            };
            uint32_t ret = sd_ble_gap_phy_update(p_gap_evt->conn_handle, &phys);
            APP_ERROR_CHECK(ret);
        } break;

        case BLE_GAP_EVT_SEC_PARAMS_REQUEST: {
            TY_PRINTF("sec param request.");
            // Pairing not supported
            uint32_t ret = sd_ble_gap_sec_params_reply(p_gap_evt->conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
            APP_ERROR_CHECK(ret);
        } break;

        case BLE_GATTS_EVT_SYS_ATTR_MISSING: {
            TY_PRINTF("sttr missing request.");
            // No system attributes have been stored.
            uint32_t ret = sd_ble_gatts_sys_attr_set(p_gap_evt->conn_handle, NULL, 0, 0);
            APP_ERROR_CHECK(ret);
        } break;

        case BLE_GATTC_EVT_TIMEOUT: {
            // Disconnect on GATT Client timeout event.
            TY_PRINTF("GATT Client Timeout.");
            uint32_t ret = sd_ble_gap_disconnect(p_gap_evt->conn_handle, BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(ret);
        } break;

        case BLE_GATTS_EVT_TIMEOUT: {
            //connect timeout
            if(p_gap_evt->params.timeout.src == BLE_GAP_TIMEOUT_SRC_CONN) {
            }
        } break;

        case BLE_GAP_EVT_CONN_PARAM_UPDATE: {
            TY_PRINTF("BLE_GAP_EVT_CONN_PARAM_UPDATE: min-%dms, max-%dms, latency-%d, timeout-%dms", \
                (uint16_t)(p_gap_evt->params.conn_param_update.conn_params.min_conn_interval*1.25),     \
                (uint16_t)(p_gap_evt->params.conn_param_update.conn_params.max_conn_interval*1.25),     \
                (uint16_t)(p_gap_evt->params.conn_param_update.conn_params.slave_latency),              \
                (uint16_t)(p_gap_evt->params.conn_param_update.conn_params.conn_sup_timeout*10) );
        } break;

        case BLE_GAP_EVT_CONN_PARAM_UPDATE_REQUEST: {
            TY_PRINTF("BLE_GAP_EVT_CONN_PARAM_UPDATE_REQUEST");
            // Accepting parameters requested by peer.
            uint32_t ret = sd_ble_gap_conn_param_update(p_gap_evt->conn_handle,
                                                    &p_gap_evt->params.conn_param_update_request.conn_params);
//            APP_ERROR_CHECK(ret);
        } break;

        default: {
        } break;
    }
}
//init SoftDeviceBLE Stack register BLE event callback
static void ty_ble_stack_init(void)
{
    uint32_t ret;

    //enable SoftDevice
    ret = nrf_sdh_enable_request();
    APP_ERROR_CHECK(ret);

    //Set default Stack SettingsGet APP RAM start address
    uint32_t ram_start = 0;
    ret = nrf_sdh_ble_default_cfg_set(TY_BLE_CONN_CFG_TAG, &ram_start);
    APP_ERROR_CHECK(ret);

    //enable BLE stack.
    ret = nrf_sdh_ble_enable(&ram_start);
    APP_ERROR_CHECK(ret);
    
    //register BLE Event handler
    NRF_SDH_BLE_OBSERVER(ty_ble_observer, 3, ty_ble_evt_handler, NULL);
}
//set default conn param
static void ty_ble_set_default_conn_param(void)
{
    ble_gap_conn_params_t param;
    memset(&param, 0, sizeof(ble_gap_conn_params_t));
    param.min_conn_interval = MSEC_TO_UNITS(20, UNIT_1_25_MS);
    param.max_conn_interval = MSEC_TO_UNITS(20, UNIT_1_25_MS);
    param.slave_latency     = 0;
    param.conn_sup_timeout  = MSEC_TO_UNITS(5000, UNIT_10_MS);
    
    uint32_t ret;
    ret = sd_ble_gap_ppcp_set(&param);
    APP_ERROR_CHECK(ret);
}
//init GAP module
static void ty_ble_gap_params_init(void)
{
    uint32_t ret;
    ble_gap_conn_sec_mode_t sec_mode;

    //security mode
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
    //device name
    ret = sd_ble_gap_device_name_set(&sec_mode, (const uint8_t *)"Demo_name", strlen("Demo_name"));
    APP_ERROR_CHECK(ret);

    //device appearance
    ret = sd_ble_gap_appearance_set(BLE_APPEARANCE_UNKNOWN);
    APP_ERROR_CHECK(ret);

    ty_ble_set_default_conn_param();
}
//Function for handling events from the GATT library.
static void ty_ble_gatt_evt_handler(nrf_ble_gatt_t* p_gatt, nrf_ble_gatt_evt_t const* p_evt)
{
    if(p_evt->evt_id == NRF_BLE_GATT_EVT_ATT_MTU_UPDATED) {
//        uint16_t m_ble_max_data_len = p_evt->params.att_mtu_effective - OPCODE_LENGTH - HANDLE_LENGTH;
        TY_PRINTF("Mtu is set to 0x%X(%d)", p_evt->params.att_mtu_effective, p_evt->params.att_mtu_effective);
    }
    if(p_evt->evt_id == NRF_BLE_GATT_EVT_DATA_LENGTH_UPDATED) {
        TY_PRINTF("Data len is set to 0x%d", p_evt->params.data_length);
    }
}
//init GATT module
static void ty_ble_gatt_init(void)
{
    uint32_t ret;
    ret = nrf_ble_gatt_init(&m_gatt, ty_ble_gatt_evt_handler);
    APP_ERROR_CHECK(ret);

    //Set MTU
//    ret = nrf_ble_gatt_att_mtu_periph_set(&m_gatt, 128);
//    APP_ERROR_CHECK(ret);
}
static void ty_ble_svc_receive_data(ble_nus_evt_t* p_evt)
{
    if(p_evt->type == BLE_NUS_EVT_RX_DATA) {
        ty_ble_receive_data_handler((void*)p_evt->params.rx_data.p_data, p_evt->params.rx_data.length);
    }
}
static void ty_ble_svc_init(void)
{
    // Initialize NUS.
    ble_nus_init_t nus_init;
    memset(&nus_init, 0, sizeof(nus_init));
    nus_init.data_handler = ty_ble_svc_receive_data;
    
    uint32_t ret;
    ret = ble_nus_init(&m_nus, &nus_init);
    APP_ERROR_CHECK(ret);
}
static void ty_ble_init_mac(void)
{
    uint8_t tmp_mac_str[12] = TY_DEVICE_MAC;
    ty_ble_mac_t mac;
    
    mac.type = 0;
    tuya_ble_str_to_hex(tmp_mac_str, 12, mac.addr);
    ty_ble_set_mac(&mac);
}
static void ty_ble_adv_init(void)
{
    m_adv_params.interval           = MSEC_TO_UNITS(s_adv_param.adv_interval_max, UNIT_0_625_MS);
    m_adv_params.duration           = MSEC_TO_UNITS(BLE_GAP_ADV_TIMEOUT_GENERAL_UNLIMITED, UNIT_10_MS);
    m_adv_params.properties.type    = s_adv_param.adv_type;
    m_adv_params.primary_phy        = 0x01;
    m_adv_params.secondary_phy      = 0x01;
    m_adv_params.filter_policy      = BLE_GAP_ADV_FP_ANY;

    m_adv_data.adv_data.len         = s_adv_data.len;
    m_adv_data.adv_data.p_data      = s_adv_data.value;
    m_adv_data.scan_rsp_data.len    = s_scan_rsp.len;
    m_adv_data.scan_rsp_data.p_data = s_scan_rsp.value;

    uint32_t ret = sd_ble_gap_adv_set_configure(&m_adv_handle, &m_adv_data, &m_adv_params);
    APP_ERROR_CHECK(ret);
}
uint32_t ty_ble_init(void)
{
    ty_ble_stack_init();
    ty_ble_gap_params_init();
    ty_ble_gatt_init();
//    ty_ble_init_mac();
    ble_conn_state_init();
    ty_ble_adv_init();
    ty_ble_svc_init();
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_start_adv(void)
{
    uint32_t ret = sd_ble_gap_adv_start(m_adv_handle, TY_BLE_CONN_CFG_TAG);
    if(ret == NRF_SUCCESS) {
        TY_PRINTF("start adv");
        is_advertising = true;
    }
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_restart_adv(void)
{
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_stop_adv(void)
{
    uint32_t ret = sd_ble_gap_adv_stop(m_adv_handle);
    if(ret == NRF_SUCCESS) {
        TY_PRINTF("stop adv");
        is_advertising = false;
    }
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_advdata_and_scanrsp(const ty_ble_data_t* p_adv_data, const ty_ble_data_t* p_scan_rsp)
{
    bool adv_flag = false;

    if(is_advertising) {
        adv_flag = true;
        ty_ble_stop_adv();
    }

    if(p_adv_data != NULL) {
        m_adv_data.adv_data.len         = p_adv_data->len;
        m_adv_data.adv_data.p_data      = (void*)p_adv_data->value;
    }
    if(p_scan_rsp != NULL) {
        m_adv_data.scan_rsp_data.len    = p_scan_rsp->len;
        m_adv_data.scan_rsp_data.p_data = (void*)p_scan_rsp->value;
    }

    uint32_t ret = sd_ble_gap_adv_set_configure(&m_adv_handle, &m_adv_data, NULL);

    if(adv_flag) {
        ty_ble_start_adv();
    }
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_adv_param(const ty_ble_adv_param_t* p_param)
{
    bool adv_flag = false;

    if(is_advertising) {
        adv_flag = true;
        ty_ble_stop_adv();
    }

    m_adv_params.interval           = MSEC_TO_UNITS(p_param->adv_interval_max, UNIT_0_625_MS);
    m_adv_params.duration           = MSEC_TO_UNITS(BLE_GAP_ADV_TIMEOUT_GENERAL_UNLIMITED, UNIT_10_MS);
    m_adv_params.properties.type    = p_param->adv_type;
    m_adv_params.primary_phy        = 0x01;
    m_adv_params.secondary_phy      = 0x01;
    m_adv_params.filter_policy      = BLE_GAP_ADV_FP_ANY;

    uint32_t ret = sd_ble_gap_adv_set_configure(&m_adv_handle, &m_adv_data, &m_adv_params);

    if(adv_flag) {
        ty_ble_start_adv();
    }
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_connect(const ty_ble_mac_t* p_mac)
{
    ble_gap_addr_t mac = {0};
    mac.addr_type = p_mac->type;
    memcpy(mac.addr, p_mac->addr, 6);
    
    ble_gap_scan_params_t scan_param;
    memset(&scan_param, 0, sizeof(ble_gap_scan_params_t));
    scan_param.interval       = MSEC_TO_UNITS(5, UNIT_0_625_MS);
    scan_param.window         = MSEC_TO_UNITS(5, UNIT_0_625_MS);
    scan_param.timeout        = MSEC_TO_UNITS(10, UNIT_10_MS);
    scan_param.active         = 1;
    scan_param.filter_policy  = BLE_GAP_SCAN_FP_ACCEPT_ALL;
    scan_param.scan_phys      = BLE_GAP_PHY_1MBPS;
    
    ble_gap_conn_params_t conn_params;
    memset(&conn_params, 0, sizeof(ble_gap_conn_params_t));
    conn_params.min_conn_interval = MSEC_TO_UNITS(20, UNIT_1_25_MS);
    conn_params.max_conn_interval = MSEC_TO_UNITS(20, UNIT_1_25_MS);
    conn_params.slave_latency     = 0;
    conn_params.conn_sup_timeout  = MSEC_TO_UNITS(5000, UNIT_10_MS);
    
    return sd_ble_gap_connect(&mac, &scan_param, &conn_params, TY_BLE_CONN_CFG_TAG);
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_reconnect(const ty_ble_mac_t* p_mac)
{
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_disconnect(void)
{
    uint32_t ret = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_conn_param(uint16_t cMin, uint16_t cMax, uint16_t latency, uint16_t timeout)
{
    ble_gap_conn_params_t gap_conn_params;
    memset(&gap_conn_params, 0, sizeof(gap_conn_params));
    gap_conn_params.min_conn_interval = MSEC_TO_UNITS(cMin, UNIT_1_25_MS);
    gap_conn_params.max_conn_interval = MSEC_TO_UNITS(cMax, UNIT_1_25_MS);
    gap_conn_params.slave_latency     = latency;
    gap_conn_params.conn_sup_timeout  = MSEC_TO_UNITS(timeout, UNIT_10_MS);
    
    return sd_ble_gap_conn_param_update(m_conn_handle, &gap_conn_params);
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_mac(const ty_ble_mac_t* p_mac)
{
    bool adv_flag = false;

    if(is_advertising) {
        adv_flag = true;
        ty_ble_stop_adv();
    }

    ble_gap_addr_t addr = {0};
    addr.addr_type = p_mac->type;
    memcpy(addr.addr, p_mac->addr, 6);
    
    uint32_t ret = sd_ble_gap_addr_set(&addr);

    if(adv_flag) {
        ty_ble_start_adv();
    }
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_get_mac(ty_ble_mac_t* p_mac)
{
    ble_gap_addr_t addr = {0};
    
    uint32_t ret = sd_ble_gap_addr_get(&addr);
    
    p_mac->type = addr.addr_type;
    memcpy(p_mac->addr, addr.addr, 6);
    
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_connect_handler(void)
{
    is_advertising = false;
    tuya_ble_connected_handler();
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_disconnect_handler(void)
{
    tuya_ble_disconnected_handler();
    tuya_ble_ota_disconn_handler();
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_receive_data_handler(const uint8_t* buf, uint32_t size)
{
//    TY_PRINTF("ble rx size: %d", size);
    tuya_ble_gatt_receive_data((void*)buf, size);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_send_data(const uint8_t* buf, uint32_t size)
{
    ret_code_t err_code;
    uint32_t tmp_count = 100;
    do
    {
        err_code = ble_nus_data_send(&m_nus, (void*)buf, (void*)&size, m_conn_handle);
        if ((err_code != NRF_ERROR_INVALID_STATE) &&
            (err_code != NRF_ERROR_RESOURCES) &&
            (err_code != NRF_ERROR_NOT_FOUND) &&
            (err_code != NRF_SUCCESS))
        {
            TY_PRINTF("ble_nus_data_send err_code = %x\n", err_code);
            APP_ERROR_CHECK(err_code);
        }
        ty_system_delay_ms(5);
        tmp_count--;
    } while (err_code == NRF_ERROR_RESOURCES && tmp_count > 0);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_get_rssi(int8_t* p_rssi)
{
    uint8_t ch_index = 0;
    return sd_ble_gap_rssi_get(m_conn_handle, p_rssi, &ch_index);
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_tx_power(int8_t tx_power)
{
    return sd_ble_gap_tx_power_set(BLE_GAP_TX_POWER_ROLE_ADV, m_adv_handle, 0);
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_device_name(const uint8_t* buf, uint16_t size)
{
    ble_gap_conn_sec_mode_t mode = {0};
    return sd_ble_gap_device_name_set(&mode, buf, size);
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_get_device_name(uint8_t* buf, uint16_t* p_size)
{
    return sd_ble_gap_device_name_get(buf, p_size);
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_dle(void)
{
    uint32_t ret;
    ble_opt_t opt;
    memset(&opt, 0x00, sizeof(opt));
    opt.common_opt.conn_evt_ext.enable = true;
    ret = sd_ble_opt_set(BLE_COMMON_OPT_CONN_EVT_EXT, &opt);
    return ret;
}






