#include "ty_flash.h"
#include "ty_system.h"
#include "nrf_fstorage.h"
#include "nrf_fstorage_sd.h"
#include "nrf_pwr_mgmt.h"
#include "app_error.h"




/*********************************************************************
 * LOCAL CONSTANT
 */
            
/*********************************************************************
 * LOCAL STRUCT
 */

/*********************************************************************
 * LOCAL VARIABLE
 */
static void ty_flash_fstorage_evt_handler(nrf_fstorage_evt_t* p_evt);
NRF_FSTORAGE_DEF(nrf_fstorage_t ty_flash_fstorage) =
{
    .evt_handler = ty_flash_fstorage_evt_handler,
    .start_addr = 0x46000,
    .end_addr   = 0x78000,
};
static __ALIGN(sizeof(uint32_t)) uint8_t ty_flash_cache[256];

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */




/*********************************************************
FN: 
*/
static void ty_flash_fstorage_evt_handler(nrf_fstorage_evt_t* p_evt)
{
    if (p_evt->result != NRF_SUCCESS) {
        TY_PRINTF("--> Event received: ERROR while executing an fstorage operation.", p_evt->len, p_evt->addr);
        return;
    }

    switch (p_evt->id)
    {
        case NRF_FSTORAGE_EVT_WRITE_RESULT: {
//            TY_PRINTF("--> Event received: wrote %d bytes at address 0x%x.", p_evt->len, p_evt->addr);
        } break;

        case NRF_FSTORAGE_EVT_ERASE_RESULT: {
//            TY_PRINTF("--> Event received: erased %d page from address 0x%x.", p_evt->len, p_evt->addr);
        } break;

        default: {
        } break;
    }
}
static void ty_flash_wait_for_ready(void)
{
    //While fstorage is busy, sleep and wait for an event.
    while(nrf_fstorage_is_busy(&ty_flash_fstorage)) {
        nrf_pwr_mgmt_run();
    }
}

/*********************************************************
FN: 
*/
uint32_t ty_flash_init(void)
{
    //init fstorage
    nrf_fstorage_api_t* p_fs_api;
    p_fs_api = &nrf_fstorage_sd;
    
    uint32_t ret;
    ret = nrf_fstorage_init(&ty_flash_fstorage, p_fs_api, NULL);
    APP_ERROR_CHECK(ret);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_flash_read(uint32_t addr, uint8_t* buf, uint32_t size)
{
    ty_system_wdt_feed();
    
    return nrf_fstorage_read(&ty_flash_fstorage, addr, buf, size);
}

/*********************************************************
FN: 
*/
uint32_t ty_flash_write(uint32_t addr, const uint8_t* buf, uint32_t size)
{
    ty_system_wdt_feed();
    
    memcpy(ty_flash_cache, buf, size);
    
    uint32_t ret = nrf_fstorage_write(&ty_flash_fstorage, addr, ty_flash_cache, size, NULL);
    if (ret == NRF_SUCCESS) {
        ty_flash_wait_for_ready();
    } else {
        TY_PRINTF("Error: flash write err_code %d", ret);
    }
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_flash_erase(uint32_t addr, uint32_t num)
{
    ty_system_wdt_feed();
    
    uint32_t ret = nrf_fstorage_erase(&ty_flash_fstorage, addr, num, NULL);
    if (ret == NRF_SUCCESS) {
        ty_flash_wait_for_ready();
    } else {
        TY_PRINTF("Error: flash erase err_code %d", ret);
    }
    return ret;
}






