#include "ty_i2c.h"
#include "nrf_drv_twi.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "app_util_platform.h"




/*********************************************************************
 * LOCAL CONSTANT
 */
#define TY_I2C_PIN_SCL    16
#define TY_I2C_PIN_SDA    15

/*********************************************************************
 * LOCAL STRUCT
 */

/*********************************************************************
 * LOCAL VARIABLE
 */
static const nrf_drv_twi_t ty_i2c_handle = NRF_DRV_TWI_INSTANCE(0);

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */




/*********************************************************
FN: 
*/
uint32_t ty_i2c_scan_address(void)
{
    uint8_t sample_data;

    TY_PRINTF("TWI device scan started.");
    
    for (uint8_t address = 1; address <= 127; address++) {
        uint32_t ret = nrf_drv_twi_rx(&ty_i2c_handle, address, &sample_data, sizeof(sample_data));

        if (ret == NRF_SUCCESS) {
            TY_PRINTF("TWI-i2c device detected at address 0x%x.", address);
            return 0;
        }
    }
 
    TY_PRINTF("No device was found.");
    
    TY_PRINTF("TWI device scan ended.");

    return 1;
}

/*********************************************************
FN: 
*/
uint32_t ty_i2c_init(void)
{
    uint32_t ret;
	
    const nrf_drv_twi_config_t twiConfig = {
        .scl                = TY_I2C_PIN_SCL,
        .sda                = TY_I2C_PIN_SDA,
        .frequency          = NRF_DRV_TWI_FREQ_400K,
        .interrupt_priority = APP_IRQ_PRIORITY_HIGH,
        .clear_bus_init     = false
    };
    
    ret = nrf_drv_twi_init(&ty_i2c_handle, &twiConfig, NULL, NULL);
    APP_ERROR_CHECK(ret);
    
//    ty_i2c_scan_address();
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_i2c_start(void)
{
    nrf_drv_twi_enable(&ty_i2c_handle);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_i2c_stop(void)
{
    nrf_drv_twi_disable(&ty_i2c_handle);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_i2c_control(uint8_t cmd, void* arg)
{
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_i2c_uninit(void)
{
    nrf_drv_twi_uninit(&ty_i2c_handle);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_i2c_send(const uint8_t addr, const uint8_t* buf, uint32_t size)
{
    uint32_t err_code;
    uint8_t retry_num = 50;
    do{
        err_code = nrf_drv_twi_tx(&ty_i2c_handle, addr, buf, size, false);
        retry_num--;
    } while((NRF_SUCCESS != err_code) && (0 < retry_num));
    return 0;
}




/*********************************************************************
 * LOCAL CONSTANT
 */
#define DELAY_CNT 1
enum {
	GPIO_INPUT_MODE,
	GPIO_OUTPUT_MODE,
};

/*********************************************************************
 * LOCAL STRUCT
 */

/*********************************************************************
 * LOCAL VARIABLE
 */

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */




/*********************************************************
FN: 
*/
void i2c_delay(unsigned long tim_1us)
{
	nrf_delay_us(tim_1us);
//	 __NOP();
}

static void i2c_sda_pin_mode_set(uint8_t mode, uint8_t level)
{
	if(mode == GPIO_INPUT_MODE){
		nrf_gpio_cfg_input(TY_I2C_PIN_SDA, NRF_GPIO_PIN_PULLUP);
	} else if(mode == GPIO_OUTPUT_MODE){
        nrf_gpio_cfg_output(TY_I2C_PIN_SDA);
	}
}

static void i2c_sda_pin_set(uint8_t level)
{
    nrf_gpio_pin_write(TY_I2C_PIN_SDA, level);
}

static void i2c_scl_pin_set(uint8_t level)
{
    nrf_gpio_pin_write(TY_I2C_PIN_SCL, level);
}

static uint8_t i2c_sda_pin_status_get(void)
{
    return nrf_gpio_pin_read(TY_I2C_PIN_SDA)?1:0;
}

/**
 * @description: i2c ack func
 * @param {type} none
 * @return: none
 */
static void i2c_ack(void)
{
    i2c_scl_pin_set(0);
    i2c_delay(DELAY_CNT);

    i2c_sda_pin_mode_set( GPIO_OUTPUT_MODE, 0 );
    i2c_sda_pin_set( 0 );
    i2c_delay(DELAY_CNT);
    
    i2c_scl_pin_set( 1 );
    i2c_delay(DELAY_CNT);
    i2c_scl_pin_set( 0 );
    i2c_delay(DELAY_CNT);
}

/**
 * @description: i2c none ack func
 * @param {type} none
 * @return: none
 */
static void i2c_noack(void)
{
    i2c_sda_pin_mode_set( GPIO_OUTPUT_MODE, 1 );
    i2c_sda_pin_set( 1 );

    i2c_delay(DELAY_CNT);
    i2c_scl_pin_set( 1 );
    i2c_delay(DELAY_CNT);
    i2c_scl_pin_set( 0 );
    i2c_delay(DELAY_CNT);
}

/**
 * @description: i2c wait ack
 * @param {type} none
 * @return: rev ack return true else return false
 */
static uint8_t i2c_wait_ack(void)
{
    uint8_t cnt = 50;

    i2c_sda_pin_mode_set( GPIO_INPUT_MODE, 1 );/* set input and release SDA */
    i2c_sda_pin_set( 1 );
    i2c_delay(DELAY_CNT);

    i2c_scl_pin_set( 0 );       /* put down SCL ready to cheack SCA status */
    i2c_delay(DELAY_CNT);
    
    i2c_scl_pin_set( 1 );
    i2c_delay(DELAY_CNT);
    
    while( i2c_sda_pin_status_get() ) /* get ack */
    {
        cnt--;
        if( cnt==0 )
        {
            i2c_scl_pin_set( 0 );
            return false;
        }
        i2c_delay(DELAY_CNT);
    }
    
    i2c_scl_pin_set( 0 );
    i2c_delay(DELAY_CNT);
    return true;
}

/**
 * @description: i2c start signal
 * @param {type} none
 * @return: none
 */
void i2c_start(void)
{
    i2c_sda_pin_mode_set( GPIO_OUTPUT_MODE, 1 );    //SDA output mode

    i2c_scl_pin_set( 1 );
    i2c_sda_pin_set( 1 );
    i2c_delay(DELAY_CNT);

    i2c_sda_pin_set( 0 );
    i2c_delay(DELAY_CNT);
  
    i2c_scl_pin_set( 0 );
    i2c_delay(DELAY_CNT);
}

/**
 * @description: i2c stop signal
 * @param {type} none
 * @return: none
 */
void i2c_stop(void)
{
    i2c_sda_pin_mode_set( GPIO_OUTPUT_MODE, 0 );     //SDA input mode

    i2c_scl_pin_set( 0 );
    i2c_sda_pin_set( 0 );
    i2c_delay(DELAY_CNT);

    i2c_scl_pin_set( 1 );
    i2c_delay(DELAY_CNT);

    i2c_sda_pin_set( 1 );
    i2c_delay(DELAY_CNT);
}

/**
 * @description: send one byte to i2c bus
 * @param {uint8_t} data send to i2c
 * @return: none
 */
void i2c_send_byte(uint8_t data)
{
    uint8_t idx = 0;
    i2c_scl_pin_set( 0 );
    i2c_sda_pin_mode_set( GPIO_OUTPUT_MODE, 1 );
    
    for( idx=0; idx<8; idx++ ) {
        if( data & 0x80 ) {
            i2c_sda_pin_set( 1 );
        } else {
            i2c_sda_pin_set( 0 );
        }
        i2c_delay(DELAY_CNT);
        
        i2c_scl_pin_set( 1 );
        i2c_delay(DELAY_CNT);
        
        i2c_scl_pin_set( 0 );
        i2c_delay(DELAY_CNT);
        
        data <<= 1;
    }
}

/**
 * @description: send bytes to i2c bus
 * @param {type} none
 * @return: none
 */
void i2c_send_bytes(uint8_t adderss_cmd, uint8_t *buff, uint8_t len)
{
    uint8_t idx;
    i2c_send_byte( adderss_cmd );
    i2c_wait_ack();

    for( idx=0; idx<len; idx++ ) {
        i2c_send_byte( buff[idx] );
        i2c_wait_ack();
    }
}

/**
 * @description: recive one byte from i2c bus
 * @param {type} none
 * @return: none
 */
void i2c_rcv_byte(uint8_t *data)
{
    uint8_t idx;
    i2c_sda_pin_mode_set( GPIO_INPUT_MODE, 1 );
    i2c_delay(25);
    
    for( idx=0; idx<8; idx++ ) {
        i2c_scl_pin_set( 0 );
        i2c_delay(DELAY_CNT);

        i2c_scl_pin_set( 1 );
        *data = *data << 1;
        if( i2c_sda_pin_status_get() ) {
            *data |= 1;
        }
        i2c_delay(DELAY_CNT);
    }
    
    i2c_scl_pin_set( 0 );
}

/**
 * @description: recive bytes from i2c bus,last byte none ack
 * @param {type} none
 * @return: none
 */
void i2c_rcv_bytes(uint8_t adderss_cmd, uint8_t *buff, uint8_t len)
{
    uint8_t idx;
    i2c_send_byte( adderss_cmd );
    i2c_wait_ack();
    
    for( idx=0; idx<len; idx++ ) {
        i2c_rcv_byte( &buff[idx] );
        
        if( idx<len-1 ) {
            i2c_ack();
        } else {
            i2c_noack();
        }
    }
}

void i2c_soft_cfg(uint8_t adderss_cmd,uint8_t reg_addr,uint8_t data)
{
    i2c_start();
    i2c_send_byte(adderss_cmd);
    i2c_wait_ack();
    i2c_send_byte(reg_addr);
    i2c_wait_ack();
    i2c_send_byte(data);
    i2c_wait_ack();		
    i2c_stop();
}

void i2c_write_reg(uint8_t address_cmd,uint8_t register_addr)
{
	i2c_start();
    i2c_send_bytes(address_cmd,&register_addr ,1);
    i2c_stop();	
}

void i2c_soft_gpio_init(void)
{
	nrf_gpio_cfg_output(TY_I2C_PIN_SDA);
	nrf_gpio_cfg_output(TY_I2C_PIN_SCL);
}











