#include "ty_pwm.h"
#include "nrf_drv_pwm.h"




/*********************************************************************
 * LOCAL CONSTANT
 */

/*********************************************************************
 * LOCAL STRUCT
 */

/*********************************************************************
 * LOCAL VARIABLE
 */
static nrf_drv_pwm_t ty_pwm = NRF_DRV_PWM_INSTANCE(0);
static nrf_drv_pwm_config_t ty_pwm_cfg = {
    .output_pins = {
        NRFX_PWM_PIN_NOT_USED | NRF_DRV_PWM_PIN_INVERTED, // channel 0
        NRFX_PWM_PIN_NOT_USED | NRF_DRV_PWM_PIN_INVERTED, // channel 1
        NRFX_PWM_PIN_NOT_USED | NRF_DRV_PWM_PIN_INVERTED, // channel 2
        NRFX_PWM_PIN_NOT_USED | NRF_DRV_PWM_PIN_INVERTED  // channel 3
    },
    .irq_priority = APP_IRQ_PRIORITY_LOWEST,
    .base_clock   = NRF_PWM_CLK_1MHz,
    .count_mode   = NRF_PWM_MODE_UP,
    .top_value    = 1000,
    .load_mode    = NRF_PWM_LOAD_COMMON,
    .step_mode    = NRF_PWM_STEP_AUTO
};
static nrf_pwm_values_common_t ty_pwm_seq_values[] = {0};
static nrf_pwm_sequence_t const ty_pwm_seq =
{
    .values.p_common = ty_pwm_seq_values,
    .length          = NRF_PWM_VALUES_LENGTH(ty_pwm_seq_values),
    .repeats         = 1000,
    .end_delay       = 0
};

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */




/*********************************************************
FN: 
*/

/*********************************************************
FN: 
*/
uint32_t ty_pwm_init(ty_pwm_t* p_pwm)
{
    ty_pwm_cfg.output_pins[0] = p_pwm->pin;
    ty_pwm_cfg.top_value = 1000000L/p_pwm->freq;
    APP_ERROR_CHECK(nrf_drv_pwm_init(&ty_pwm, &ty_pwm_cfg, NULL));
    
    if(p_pwm->polarity == 0) {
        ty_pwm_seq_values[0] = (uint16_t)(((1000000L/p_pwm->freq)*((float)p_pwm->duty/100))) | 0x8000;
    } else {
        ty_pwm_seq_values[0] = (uint16_t)(((1000000L/p_pwm->freq)*((float)p_pwm->duty/100)));
    }
    return nrf_drv_pwm_simple_playback(&ty_pwm, &ty_pwm_seq, 1, NRF_DRV_PWM_FLAG_LOOP);
}

/*********************************************************
FN: 
*/
uint32_t ty_pwm_start(ty_pwm_t* p_pwm)
{
    ty_pwm_uninit(p_pwm);
    ty_pwm_init(p_pwm);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_pwm_stop(ty_pwm_t* p_pwm)
{
    return nrf_drv_pwm_stop(&ty_pwm, false);
}

/*********************************************************
FN: 
*/
uint32_t ty_pwm_control(ty_pwm_t* p_pwm, uint8_t cmd, void* arg)
{
    switch(cmd) {
        case TY_PWM_CMD_SET_POLARITY: {
            ty_pwm_set_polarity_t* param = arg;
            p_pwm->polarity = param->polarity;
            ty_pwm_start(p_pwm);
        } break;
        
        case TY_PWM_CMD_SET_FREQ: {
            ty_pwm_set_freq_t* param = arg;
            p_pwm->polarity = param->freq;
            ty_pwm_start(p_pwm);
        } break;
        
        case TY_PWM_CMD_SET_DUTY: {
            ty_pwm_set_duty_t* param = arg;
            p_pwm->polarity = param->duty;
            ty_pwm_start(p_pwm);
        } break;
        
        default: {
        } break;
    }
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_pwm_uninit(ty_pwm_t* p_pwm)
{
    nrf_drv_pwm_uninit(&ty_pwm);
    return 0;
}







