#include "ty_spi.h"
#include "nrf_drv_spi.h"
#include "nrf_gpio.h"
#include "app_error.h"




/*********************************************************************
 * LOCAL CONSTANT
 */
#define TY_TEST_SPI_PIN_MISO         2
#define TY_TEST_SPI_PIN_MOSI         3
#define TY_TEST_SPI_PIN_CLK          4
#define TY_TEST_SPI_PIN_CSN          5

/*********************************************************************
 * LOCAL STRUCT
 */

/*********************************************************************
 * LOCAL VARIABLE
 */
static volatile bool s_transferOk = true;
static const nrf_drv_spi_t s_spiHandle = NRF_DRV_SPI_INSTANCE(1);

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */
static void spiCallbackFunc(nrf_drv_spi_evt_t const *pEvent, void *arg);




/*********************************************************
FN: 
*/
void ty_spi_init(void)
{
	nrf_gpio_cfg_output(TY_TEST_SPI_PIN_CSN);
}

/*********************************************************
FN: 
*/
void ty_spi_csn_set(bool pinState)
{
    if(pinState == TY_SPI_CS_OFF) {
		nrf_gpio_pin_write(TY_TEST_SPI_PIN_CSN, 1);
    } else if(pinState == TY_SPI_CS_ON) {
        nrf_gpio_pin_write(TY_TEST_SPI_PIN_CSN, 0);
    }
}

/*********************************************************
FN: 
*/
void ty_spi_readWriteData(uint8_t *pWriteData, uint8_t *pReadData, uint8_t writeDataLen)
{
	s_transferOk = false;

	APP_ERROR_CHECK(nrf_drv_spi_transfer(&s_spiHandle, pWriteData, writeDataLen, pReadData, writeDataLen));

	while(!s_transferOk) {
		__WFE();
	}
}

/*********************************************************
FN: Turn on SPI, which is different from initialization: CS pin is not initialized
*/
void ty_spi_enable(void)
{
	ret_code_t ret;
	
	nrf_drv_spi_config_t spiConfig = NRF_DRV_SPI_DEFAULT_CONFIG;
	//Configure the SPI port, pay attention not to set the CSN here, and use the GPIO port to control
	spiConfig.miso_pin = TY_TEST_SPI_PIN_MISO;
	spiConfig.mosi_pin = TY_TEST_SPI_PIN_MOSI;
	spiConfig.sck_pin = TY_TEST_SPI_PIN_CLK;
	spiConfig.mode = NRF_DRV_SPI_MODE_0;
	spiConfig.frequency = SPI_FREQUENCY_FREQUENCY_K250;	
	spiConfig.irq_priority = 4; //The priority used in the timer must be less than 6
	
	ret = nrf_drv_spi_init(&s_spiHandle, &spiConfig, spiCallbackFunc, NULL);
	APP_ERROR_CHECK(ret);
}

/*********************************************************
FN: 
*/
void ty_spi_disable(void)
{
	nrf_drv_spi_uninit(&s_spiHandle);
}

/*********************************************************
FN: 
*/
static void spiCallbackFunc(nrf_drv_spi_evt_t const *pEvent, void *arg)
{
	s_transferOk = true;
}














