#include "ty_system.h"
#include "ty_ble.h"
#include "ty_uart.h"
#include "ty_flash.h"
#include "ty_timer.h"
#include "ty_rtc.h"
#include "ty_pwm.h"
#include "ty_adc.h"
#include "ty_pin.h"
#include "ty_i2c.h"
#include "ty_spi.h"
#include "tuya_ble_api.h"
#include "tuya_ble_log.h"
#include "tuya_ble_sdk_test.h"
#include "nrf_drv_wdt.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_log_ctrl.h"
#include "nrf_delay.h"
#include "ty_oled.h"




/*********************************************************************
 * LOCAL CONSTANT
 */
            
/*********************************************************************
 * LOCAL STRUCT
 */

/*********************************************************************
 * LOCAL VARIABLE
 */
static nrf_drv_wdt_channel_id m_channel_id;

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */




/*********************************************************
FN: 
*/
uint32_t ty_system_init(uint8_t location)
{
    switch(location)
    {
        case 0: {
            ty_system_log_init();
            //peripheral init
            ty_flash_init();
            ty_uart_init();
            ty_rtc_init();
        } break;
        
        case 1: {
#if TUYA_BLE_SDK_TEST
            uint32_t ty_i2c_scan_address(void);
            if(ty_i2c_scan_address() == 0) {
                ty_oled_init();
            }
            
            extern uint32_t ty_system_load_pid(void);
            ty_system_load_pid();
#endif
        } break;
        
        case 2: {
        } break;
        
        default: {
        } break;
    }
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_mainloop(void)
{
    tuya_ble_main_tasks_exec();
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_reset(void)
{
    NRF_LOG_FINAL_FLUSH();
    NVIC_SystemReset();
    return 0;
}

/*********************************************************
FN: 
*/
static void wdt_event_handler(void)
{
    //NOTE: The max amount of time we can spend in WDT interrupt is two cycles of 32768[Hz] clock - after that, reset occurs
}
uint32_t ty_system_wdt_init(void)
{
//    uint32_t ret = NRF_SUCCESS;
//    //Configure WDT.
//    nrf_drv_wdt_config_t config = NRF_DRV_WDT_DEAFULT_CONFIG;
//    ret = nrf_drv_wdt_init(&config, wdt_event_handler);
//    APP_ERROR_CHECK(ret);
//    ret = nrf_drv_wdt_channel_alloc(&m_channel_id);
//    APP_ERROR_CHECK(ret);
//    nrf_drv_wdt_enable();
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_wdt_uninit(void)
{
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_wdt_feed(void)
{
//    nrf_drv_wdt_channel_feed(m_channel_id);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_log_init(void)
{
#if (TY_LOG_ENABLE||TUYA_BLE_LOG_ENABLE||TUYA_APP_LOG_ENABLE)
    elog_init();
//    elog_set_fmt(ELOG_LVL_DEBUG, ELOG_FMT_LVL);
    elog_start();
#endif
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_log_hexdump(const char *name, uint8_t width, uint8_t *buf, uint16_t size)
{
#if (TY_LOG_ENABLE||TUYA_BLE_LOG_ENABLE||TUYA_APP_LOG_ENABLE)
    elog_hexdump(name, width, buf, size);
#endif
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_delay_ms(uint32_t ms)
{
    nrf_delay_ms(ms);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_delay_us(uint32_t us)
{
    nrf_delay_us(us);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_enter_sleep(void)
{
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_exit_sleep(void)
{
    return 0;
}

/*********************************************************
FN: 
*/
bool ty_system_is_sleep(void)
{
    return 0;
}

#if TUYA_BLE_SDK_TEST
#if CHIP_PLATFORM_NRF52832
/*********************************************************
FN: 
*/
uint32_t ty_system_set_pid(uint8_t* buf, uint32_t size)
{
    tuya_ble_nv_erase(BOARD_FLASH_TEST_PID_ADDR, 0x1000);
    tuya_ble_nv_write(BOARD_FLASH_TEST_PID_ADDR, buf, size+2);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_get_pid(uint8_t* buf, uint32_t* p_size)
{
    uint8_t tmp_buf[TUYA_BLE_PRODUCT_ID_MAX_LEN+2] = {0};
    tuya_ble_nv_read(BOARD_FLASH_TEST_PID_ADDR, tmp_buf, TUYA_BLE_PRODUCT_ID_MAX_LEN+2);
    
    if(tmp_buf[1] == 0xFF) {
        *p_size = TUYA_BLE_PRODUCT_ID_MAX_LEN;
    } else {
        *p_size = tmp_buf[1];
    }
    
    memcpy(buf, &tmp_buf[2], tmp_buf[1]);
    
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_load_pid(void)
{
    uint8_t tmp_buf[TUYA_BLE_PRODUCT_ID_MAX_LEN+2] = {0};
    tuya_ble_nv_read(BOARD_FLASH_TEST_PID_ADDR, tmp_buf, TUYA_BLE_PRODUCT_ID_MAX_LEN+2);
    if(tmp_buf[0] != 0xFF) {
        tuya_ble_device_update_product_id(tmp_buf[0], tmp_buf[1], &tmp_buf[2]);
    }
    return 0;
}
#endif
#endif






