#include "ty_timer.h"
#include "app_timer.h"




/*********************************************************************
 * LOCAL CONSTANT
 */
#define TY_TIMER_MAX_NUM                    20
            
/*********************************************************************
 * LOCAL STRUCT
 */
//#pragma pack(1)
typedef struct {
    app_timer_t data;
    uint8_t     is_occupy;
    uint32_t    ms;
} ty_timer_item_t;
//#pragma pack()

/*********************************************************************
 * LOCAL VARIABLE
 */
static ty_timer_item_t ty_timer_pool[TY_TIMER_MAX_NUM] = {0};

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */




/*********************************************************
FN: 
*/
static app_timer_id_t acquire_timer(uint32_t ms)
{
    for(uint8_t i=0; i<TY_TIMER_MAX_NUM; i++) {
        if (ty_timer_pool[i].is_occupy == 0) {
            ty_timer_pool[i].is_occupy = 1;
            ty_timer_pool[i].ms = ms;
            return (void*)&ty_timer_pool[i].data;
        }
    }
    return NULL;
}

/*********************************************************
FN: 
*/
static int32_t release_timer(void* timer_id)
{
    for(uint8_t i=0; i<TY_TIMER_MAX_NUM; i++) {
        if (timer_id == &ty_timer_pool[i].data) {
            ty_timer_pool[i].is_occupy = 0;
            return i;
        }
    }
    return -1;
}

/*********************************************************
FN: 
*/
static int32_t find_timer_ms(void* timer_id, uint32_t *ms)
{
    for(uint8_t i=0; i<TY_TIMER_MAX_NUM; i++) {
        if (timer_id == &ty_timer_pool[i].data) {
            *ms = ty_timer_pool[i].ms;
            return i;
        }
    }
    return -1;
}

/*********************************************************
FN: 
*/
uint32_t ty_timer_create(void** p_timer_id, uint32_t ms, ty_timer_mode_t mode, ty_timer_handler_t handler)
{
//    static bool is_init = false;
//    if(!is_init) {
//        uint32_t ret = app_timer_init();
//        is_init = (ret == NRF_SUCCESS);
//    }
    
    app_timer_id_t id = acquire_timer(ms);
    if(id == NULL) {
        TY_PRINTF("Error: App timer is used up");
        return NRF_ERROR_NO_MEM;
    }

    ty_timer_handler_t timer_handler = handler;
    uint32_t ret = app_timer_create(&id, (app_timer_mode_t)mode, timer_handler);
    APP_ERROR_CHECK(ret);
    
    *p_timer_id = id;
    
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_timer_delete(void* timer_id)
{
    int32_t id = release_timer(timer_id);
    if(id == -1) {
        TY_PRINTF("Error: Timer id is not valid");
        return NRF_ERROR_INVALID_PARAM;
    }

    return app_timer_stop(timer_id);
}

/*********************************************************
FN: 
*/
uint32_t ty_timer_start(void* timer_id)
{
    uint32_t find_ms;
    
    if(find_timer_ms(timer_id, &find_ms) >= 0) {
        return app_timer_start(timer_id, APP_TIMER_TICKS(find_ms), NULL);
    } else {
        TY_PRINTF("Error: Timer id is not found");
        return NRF_ERROR_NOT_FOUND;
    }
}

/*********************************************************
FN: 
*/
uint32_t ty_timer_stop(void* timer_id)
{
    return app_timer_stop(timer_id);
}

/*********************************************************
FN: 
*/
uint32_t ty_timer_restart(void* timer_id, uint32_t ms)
{
    uint32_t find_ms;
    
    if(find_timer_ms(timer_id, &find_ms) >= 0) {
        uint32_t ret;
        ret = app_timer_stop((app_timer_id_t)timer_id);
        ret = app_timer_start((app_timer_id_t)timer_id, APP_TIMER_TICKS(ms), NULL);
        return ret;
    } else {
        TY_PRINTF("Error: not find_timer_ms");
        return NRF_ERROR_NOT_FOUND;
    }
}









